package com.dsic.gj_erp.controller.kban;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.annotation.AuthFunction;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.jcsj.DmBzry;
import com.dsic.gj_erp.bean.jhgk.DmSygdMx;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.jhgk.DmYdjhxf;
import com.dsic.gj_erp.exception.ServiceException;
import com.dsic.gj_erp.service.KbanService;
import com.dsic.gj_erp.service.jcsj.DmBzryService;
import com.dsic.gj_erp.service.jhgk.DmSygdMxService;
import com.dsic.gj_erp.service.sy.impl.SyMsgService;
import com.dsic.gj_erp.service.zyjh.ZyjhService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 各作业区划分规则
 * 一区:上料
 * 二区:一垮,三跨 切割
 * 三区:二跨
 * 四区:四五六垮
 * 五区:三跨 曲加工
 * 六区:光电,吊车
 * 七垮:型材
 */
@RestController
@RequestMapping("kban")
@AllArgsConstructor
public class KbanController {

    private final ZyjhService zyjhService;
    private final DmBzryService cqService;
    private final KbanService kbanService;
    private final SyMsgService msgService;
    private final DmSygdMxService service;

    /**
     * 作业区完成情况饼图
     */
    @GetMapping("/zxqwcqk")
    public ResultBean<?> getZxqwcqk(){
        List<DmYdjh> list=zyjhService.list(Wrappers.<DmYdjh>lambdaQuery().ge(DmYdjh::getZt,2));

        AtomicInteger 上料计划 = new AtomicInteger(0);
        AtomicInteger 上料完成 = new AtomicInteger(0);
        HashMap<String, Object> 上料 = new HashMap<>();

        AtomicInteger 二区计划 = new AtomicInteger(0);
        AtomicInteger 二区完成 = new AtomicInteger(0);
        HashMap<String, Object> 二区 = new HashMap<>();

        AtomicInteger 三区计划 = new AtomicInteger(0);
        AtomicInteger 三区完成 = new AtomicInteger(0);
        HashMap<String, Object> 三区 = new HashMap<>();

        AtomicInteger 四区计划 = new AtomicInteger(0);
        AtomicInteger 四区完成 = new AtomicInteger(0);
        HashMap<String, Object> 四区 = new HashMap<>();

        list.forEach(item->{
            String subStr=item.getTzbh().substring(6,7);
            int zt=0;

            try {
                zt = Integer.parseInt(subStr);
            }catch (Exception e){
                e.printStackTrace();
            }

            if (zt==11){
                上料计划.incrementAndGet();
            }

            if (zt>11){
                上料完成.incrementAndGet();
            }

            //二区
            if ("1".equals(subStr)||"3".equals(subStr)){
                if (zt>=2){
                    二区计划.incrementAndGet();
                }
                if (zt>=62){
                    二区完成.incrementAndGet();
                }
            }

            //三区
            if ("2".equals(subStr)){
                if (zt>=2){
                    三区计划.incrementAndGet();
                }
                if (zt>=62){
                    三区完成.incrementAndGet();
                }
            }

            //四区
            if ("4".equals(subStr)){
                if (zt>=2){
                    四区计划.incrementAndGet();
                }
                if (zt>=62){
                    四区完成.incrementAndGet();
                }
            }
        });

        ArrayList<Object> result = new ArrayList<>();
        上料.put("zyq","一区");
        上料.put("wcl",上料计划.get()-上料完成.get());
        上料.put("wc",上料完成.get());
        result.add(上料);

        二区.put("zyq","二区");
        二区.put("wcl",二区计划.get()-二区完成.get());
        二区.put("wc",二区完成.get());
        result.add(二区);

        三区.put("zyq","三区");
        三区.put("wcl",三区计划.get()-三区完成.get());
        三区.put("wc",三区完成.get());
        result.add(三区);

        四区.put("zyq","四区");
        四区.put("wcl",四区计划.get()-四区完成.get());
        四区.put("wc",四区完成.get());
        result.add(四区);

        return new ResultBean<>(list);
    }

    /**
     * 作业区执行情况
     */
    @GetMapping("/zxqZxDetail")
    public ResultBean<?> zxqZxDetail(){
        return new ResultBean<>(this.kbanService.zxqZxDetail());
    }

    /**
     * 作业计划
     */
    @GetMapping("/zyjh")
    public ResultBean<?> getZyjhtj(String zyq){
        return new ResultBean<>(this.kbanService.getZyjhtj(zyq));
    }

    @GetMapping("/ydjh")
    public ResultBean<?> getList(){
        DateTime date = DateUtil.date();
        String from = DateUtil.beginOfMonth(date).toString("yyyy/MM/dd");
        String to = DateUtil.endOfMonth(date).toString("yyyy/MM/dd");
        List<DmYdjhxf> list=  this.service.tqSygd1(from,to);
        return new ResultBean<>(list);
    }

    /**
     * 消息
     */
    @GetMapping("msg")
    public ResultBean<?> getMsg(){
        return new ResultBean<>(msgService.list());
    }

    /**
     * 获取所有未处理批次信息
     */
    @GetMapping("wcl")
    public ResultBean<?> getWcl(){
        return new ResultBean<>(zyjhService.getWcl());
    }

    /**
     * 看板线表计划相关数据
     */
    @GetMapping("xbjh")
    public ResultBean<?> kban(String nf){
        if (ObjUtil.isEmpty(nf)){
            throw new ServiceException(10001,"年份不能为空");
        }

        List<Double> xbjh=kbanService.getEveryMonXbzlByNf(nf);
        List<Map<String, Object>> comp = kbanService.getEveryCompZlByNf(nf);

        HashMap<Object, Object> map = new HashMap<>();
        map.put("xbjh", xbjh);
        map.put("ydjh", comp);
        return new ResultBean<>(map);
    }

    /**
     * 计划跟踪
     */
    @GetMapping("getJhgz")
    public ResultBean<?> getJhgz(){
        return new ResultBean<>(kbanService.getJhgz());
    }

    /**
     * 按照设备编号查询作业计划
     */
    @GetMapping("getZyjhDetailBySbCode")
    public ResultBean<?> getZyjhDetailBySbCode(String code){
        if (ObjUtil.isEmpty(code)){
            throw new ServiceException(10001,"设备编号不能为空");
        }
        return new ResultBean<>(kbanService.getZyjhDetailBySbCode(code));
    }

    /**
     * 统计划线、切割设备本人计划及作业成功
     */
    @GetMapping("getSb")
    public ResultBean<?> getSb(){
        List<Map<String, Object>> sb = kbanService.getSb();
        return new ResultBean<>(sb);
    }

    /**
     * 按照人员工号查询当月所有参与的任务
     */
    @GetMapping("getZjjhByUserCode")
    public ResultBean<?> getZjjhByUserCode(String userCode){
        if (ObjUtil.isEmpty(userCode)){
            throw new ServiceException(10001,"用户不能为空");
        }
        return new ResultBean<>(kbanService.getZjjhByUserCode(userCode));
    }

    /**
     * 按照跨位查询作业计划明细
     */
    @GetMapping("getKwDetailByKw")
    public ResultBean<?> getEveryKw(String kw){
        if (ObjUtil.isEmpty(kw)){
            throw new ServiceException(10001,"垮位不能为空");
        }

        QueryWrapper<DmYdjh> wrapper = new QueryWrapper<>();
        if ("1".equals(kw)){
            wrapper.lambda().ge(DmYdjh::getZt, 11);
        }

        if (Arrays.asList("2","3","4").contains(kw)){
            wrapper.eq("SUBSTRING(tzbh,6,1)", kw);
            wrapper.ge("zt", "42");
        }
        return new ResultBean<>(zyjhService.list(wrapper));
    }

    /**
     * 本月各跨位情况
     */
    @GetMapping("getEveryKw")
    public ResultBean<?> getEveryKw(){
        Map<String, Object> map = kbanService.getEveryKw();
        return new ResultBean<>(map);
    }

    /**
     * 在建船只,各船只进度
     */
    @GetMapping("gcjd")
    public ResultBean<?> gcjd(){
        return new ResultBean<>(zyjhService.gcjd());
    }

    /**
     * 各垮计划完成情况
     */
    @GetMapping("gxjhwcqk")
    public ResultBean<?> gxjhwcqk(){
        List<Map<String, Object>> gxjhwcqk = zyjhService.gxjhwcqk();
        return new ResultBean<>(gxjhwcqk);
    }

    /**
     * 各月份产量(吨)
     */
    @GetMapping("gyfcl")
    public ResultBean<?> gyfcl(){
        return new ResultBean<>(zyjhService.gyfcl());
    }

    /**
     * 计划完成情况(吨)
     */
    @GetMapping("jhwcqk")
    public ResultBean<?> jhwcqk(){
        return new ResultBean<>(zyjhService.jhwcqk());
    }

    /**
     * 各垮产量
     */
    @GetMapping("gekuachaliang")
    public ResultBean<?> gekuachaliang(){
        return new ResultBean<>(zyjhService.gekuachaliang());
    }

    /**
     * 考勤情况
     */
    @GetMapping("kq")
    public ResultBean<?> kq(){
        List<DmBzry> ryList=cqService.list(Wrappers.<DmBzry>lambdaQuery()
                .isNotNull(DmBzry::getUserCode).isNotNull(DmBzry::getZt).ne(DmBzry::getZt,""));

        List<DmBzry> collect = ryList.stream().filter(item -> "01".equals(item.getZt())).collect(Collectors.toList());

        Map<String,Object> map=new HashMap<>();
        map.put("total",ryList.size());
        map.put("rows",ryList);
        map.put("bb", collect.size());
        map.put("eb",ryList.size()-collect.size());
        return new ResultBean<>(map);
    }
}
